package com.chelvc.cloud.maintain.controller;

import com.chelvc.cloud.maintain.copier.ArticleCopier;
import com.chelvc.cloud.maintain.vo.ArticleVO;
import com.chelvc.cloud.vehicle.api.dto.ArticleDTO;
import com.chelvc.cloud.vehicle.api.param.ArticleModifyParam;
import com.chelvc.cloud.vehicle.api.param.ArticlePagingParam;
import com.chelvc.cloud.vehicle.api.service.ArticleService;
import com.chelvc.framework.base.annotation.ResponseWrapping;
import com.chelvc.framework.base.util.ResourceUtils;
import com.chelvc.framework.common.model.Pagination;
import org.apache.dubbo.config.annotation.DubboReference;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.Min;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 文章接口
 *
 * @author xp
 * @data 2023/4/4
 */
@Validated
@RestController
@ResponseWrapping
public class ArticleController {

    @DubboReference
    private ArticleService articleService;

    /**
     * 新增文章
     *
     * @param param 新增参数
     * @return 文章主键
     */
    @PostMapping("/article")
    public Long addArticle(@RequestBody @Valid ArticleModifyParam param) {
        return this.articleService.addArticle(param);
    }

    /**
     * 修改文章
     *
     * @param id    文章主键
     * @param param 修改参数
     */
    @PutMapping("/article/{id}")
    public void updateArticle(@PathVariable("id") @Min(value = 1, message = "文章主键不能小于1") Long id,
                               @RequestBody @Valid ArticleModifyParam param) {
        this.articleService.updateArticle(id, param);
    }


    /**
     * 获取文章信息
     *
     * @param id 文章主键
     * @return 文章信息
     */
    @GetMapping("/article/{id}")
    public ArticleVO getArticle(@PathVariable("id") @Min(value = 1, message = "文章主键不能小于1") Long id) {
        ArticleDTO article = this.articleService.getArticle(id);
        ResourceUtils.required(article, "文章不存在");
        return this.covert(article);
    }

    /**
     * 查询文章分页
     *
     * @param param 查询参数
     * @return 文章分页信息
     */
    @GetMapping("/article/paging")
    public Pagination<ArticleVO> getArticlePaging(@Valid ArticlePagingParam param) {
        Pagination<ArticleDTO> articlePaging = this.articleService.getArticlePaging(param);
        List<ArticleVO> articles = articlePaging.getRecords().stream().map(this::covert).collect(Collectors.toList());
        return articlePaging.convert(articles);
    }

    /**
     * 转换
     *
     * @param article
     * @return
     */
    public ArticleVO covert(ArticleDTO article) {
        return ArticleCopier.INSTANCE.copying(article);
    }

}
