package com.chelvc.cloud.maintain.controller;

import com.chelvc.cloud.maintain.copier.EnquireCopier;
import com.chelvc.cloud.maintain.vo.EnquireVO;
import com.chelvc.cloud.vehicle.api.dto.EnquireDTO;
import com.chelvc.cloud.vehicle.api.param.EnquireModifyParam;
import com.chelvc.cloud.vehicle.api.param.EnquirePagingParam;
import com.chelvc.cloud.vehicle.api.service.EnquireService;
import com.chelvc.framework.base.annotation.ResponseWrapping;
import com.chelvc.framework.base.util.ResourceUtils;
import com.chelvc.framework.common.model.Pagination;
import org.apache.dubbo.config.annotation.DubboReference;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.Min;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 询价接口
 * 
 * @author xp
 * @data 2023/4/5
 */
@Validated
@RestController
@ResponseWrapping
public class EnquireController {
    
    @DubboReference
    private EnquireService enquireService;

    /**
     * 新增询价
     *
     * @param param 新增参数
     * @return 询价主键
     */
    @PostMapping("/enquire")
    public Long addEnquire(@RequestBody @Valid EnquireModifyParam param) {
        return this.enquireService.addEnquire(param);
    }

    /**
     * 修改询价
     *
     * @param id    询价主键
     * @param param 修改参数
     */
    @PutMapping("/enquire/{id}")
    public void updateEnquire(@PathVariable("id") @Min(value = 1, message = "询价主键不能小于1") Long id,
                              @RequestBody @Valid EnquireModifyParam param) {
        this.enquireService.updateEnquire(id, param);
    }


    /**
     * 获取询价信息
     *
     * @param id 询价主键
     * @return 询价信息
     */
    @GetMapping("/enquire/{id}")
    public EnquireVO getEnquire(@PathVariable("id") @Min(value = 1, message = "询价主键不能小于1") Long id) {
        EnquireDTO enquire = this.enquireService.getEnquire(id);
        ResourceUtils.required(enquire, "询价不存在");
        return this.covert(enquire);
    }

    /**
     * 查询询价分页
     *
     * @param param 查询参数
     * @return 询价分页信息
     */
    @GetMapping("/enquire/paging")
    public Pagination<EnquireVO> getEnquirePaging(@Valid EnquirePagingParam param) {
        Pagination<EnquireDTO> enquirePaging = this.enquireService.getEnquirePaging(param);
        List<EnquireVO> enquires = enquirePaging.getRecords().stream().map(this::covert).collect(Collectors.toList());
        return enquirePaging.convert(enquires);
    }

    /**
     * 转换
     *
     * @param enquire
     * @return
     */
    public EnquireVO covert(EnquireDTO enquire) {
        return EnquireCopier.INSTANCE.copying(enquire);
    }
    
}
